
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _ENGINE_HH_
#define _ENGINE_HH_

#include <boost/shared_ptr.hpp>
#include <SDL/SDL.h>

#include "singleton.hh"
#include "dispatcher.hh"


namespace dc {


class engine_impl;
class video;

class engine : public singleton<engine>
{
public:
	engine(const std::string& name, int argc, char* argv[],
			const std::string& configFile);
	virtual ~engine();

	int run();
	void stop();

	void setTimestep(scalar ts);
	scalar getTimestep();
	void setMaxFPS(long maxfps);
	long getMaxFPS();

	video& getVideo();
	long getFPS();

	// Override these if you want.
	virtual void update(scalar t, scalar dt);
	virtual void draw(scalar alpha);
	virtual void dispatchEvent(const SDL_Event& event);

private:
	boost::shared_ptr<engine_impl> impl;
};


} // namespace dc

#endif // _ENGINE_HH_

